/*********************************************************************
 *                  Flowcode CAL Delays File
 *
 * File: AVR_CAL_Delay.c
 *
 * (c) 2011 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 060911 | BR | Created
 * 020312 | BR | Updated to v5.1 CAL Schema
 */


#include <inttypes.h>

#include <util/delay_basic.h>

static inline void delay_us(uint8_t __us) __attribute__((always_inline));
static inline void delay_10us(uint8_t __us10) __attribute__((always_inline));
static inline void delay_ms(uint8_t __ms) __attribute__((always_inline));
static inline void delay_s(uint8_t __ms) __attribute__((always_inline));


/*
static  void delay_us(uint8_t __us) __attribute__((always_inline));
static  void delay_10us(uint8_t __us10) __attribute__((always_inline));
static  void delay_ms(uint8_t __ms) __attribute__((always_inline));
static  void delay_s(uint8_t __ms) __attribute__((always_inline));
*/

// Define number of counts per time period using _delay_loop_2
#define MS_COUNT2 F_CPU / 4000
#define US10_COUNT2 F_CPU / 400000
#define US_COUNT2 F_CPU / 400000 //mim


void delay_us(uint8_t __us)
{
	_delay_loop_2((US_COUNT2 * __us)/10); //mim
	//_delay_loop_2(US_COUNT2 * __us);
}


void delay_10us(uint8_t __us10)
{
	_delay_loop_2(US10_COUNT2 * __us10);
}


void delay_ms(uint8_t __ms)
{
	uint8_t __ticks;

	for(__ticks=0;__ticks<__ms;__ticks++)
	{
		_delay_loop_2(MS_COUNT2);
	}
}


void delay_s(uint8_t __s)
{
	uint8_t __ticks;

	for(__ticks=0; __ticks<__s; __ticks++)
	{
		delay_ms(250);
		delay_ms(250);
		delay_ms(250);
		delay_ms(250);
	}
}


#define nop()	asm("NOP")


//Original Byte Delays
static void Wdt_Delay_S(char delay);
static void Wdt_Delay_Ms(char delay);

//New Int Delays
static void FCI_DELAYINT_US(MX_UINT16 Delay);

static void FCI_DELAYINT_MS(MX_UINT16 Delay);
static void FCI_DELAYINTWDT_MS(MX_UINT16 Delay);

static void FCI_DELAYINT_S(MX_UINT16 Delay);
static void FCI_DELAYINTWDT_S(MX_UINT16 Delay);



static void Wdt_Delay_S(char delay)
{
    char i;
    short j;
    for (i=0; i<delay; i++)
    {
        for (j=0; j<999; j++)
        {
            Wdt_Delay_Ms(1);
        }
    }
}

static void Wdt_Delay_Ms(char delay)
{
    char i;
    for (i=0; i<delay; i++)
    {
        MX_CLEAR_WATCHDOG;        //Clear Watchdog Timer
        delay_ms(1);
    }
}

static void FCI_DELAYINT_US(MX_UINT16 Delay)
{
  	while (Delay > 255)
  	{
	  	delay_us(255);
	  	Delay = Delay - 255;
  	}
  	if (Delay > 0)
  		delay_us(Delay & 0xFF);
}

static void FCI_DELAYINT_MS(MX_UINT16 Delay)
{
  	while (Delay > 255)
  	{
	  	delay_ms(255);
	  	Delay = Delay - 255;
  	}
  	if (Delay > 0)
  		delay_ms(Delay & 0xFF);
}

static void FCI_DELAYINTWDT_MS(MX_UINT16 Delay)
{
  	while (Delay > 255)
  	{
	  	Wdt_Delay_Ms(255);
	  	Delay = Delay - 255;
  	}
  	if (Delay > 0)
  		Wdt_Delay_Ms(Delay & 0xFF);
}

static void FCI_DELAYINT_S(MX_UINT16 Delay)
{
  	while (Delay > 255)
  	{
	  	delay_s(255);
	  	Delay = Delay - 255;
  	}
  	if (Delay > 0)
  		delay_s(Delay & 0xFF);
}

static void FCI_DELAYINTWDT_S(MX_UINT16 Delay)
{
  	while (Delay > 255)
  	{
	  	Wdt_Delay_S(255);
	  	Delay = Delay - 255;
  	}
  	if (Delay > 0)
  		Wdt_Delay_S(Delay & 0xFF);
}
